@@ -2,7 +2,7 @@
 ** xsetwacom.c
 **
 ** Copyright (C) 2003 - John E. Joganic
-** Copyright (C) 2004-2008 - Ping Cheng
+** Copyright (C) 2004-2009 - Ping Cheng
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU Lesser General Public License
@@ -33,10 +33,14 @@
 **   2007-07-24 0.1.4 - PC - Support Screen_No option
 **   2008-01-17 0.1.5 - PC - Add DISPLAYTOGGLE command
 **   2008-03-24 0.1.6 - PC - Added Touch for serial Tablet PC (ISDv4)
+**   2008-08-27 0.1.7 - PC - Added get XORGVER to xsetwacom
+**   2009-02-27 0.1.8 - PC - Added LeftOf and AboveOf to TwinView
+**   2009-05-18 0.1.9 - PC - Support get/set serial command
+**   2009-07-14 0.2.0 - PC - Support Nvidia Xinerama setting
 **
 ****************************************************************************/
 
-#define XSETWACOM_VERSION "0.1.6"
+#define XSETWACOM_VERSION "0.1.9"
 
 #include "../include/util-config.h"
 
@@ -102,8 +106,11 @@ struct _PARAMINFO
 static const char* tv_char[] = 
 {
 	"none",
+	"xinerama"
 	"vertical",
 	"horizontal",
+	"aboveof"
+	"leftof"
 	"NULL"
 };
 
@@ -301,7 +308,7 @@ static PARAMINFO gParamInfo[] =
 
 	{ "CommonDBG",
 		"Level of debugging statements applied to all devices "
-		"associated with the same tablet. default is 0 (off). ",
+		"associated with the same tablet. \n\t\t   default is 0 (off). ",
 		XWACOM_PARAM_COMMONDBG, VALUE_OPTIONAL, RANGE, 
 		0, 100, SINGLE_VALUE, 0 },
 
@@ -328,10 +335,11 @@ static PARAMINFO gParamInfo[] =
 		0, 100, PACKED_CURVE, 0x00006464},
 
 	{ "TwinView",
-		"Sets the mapping to TwinView horizontal/vertical/none. "
-		"Values = none, vertical, horizontal (default is none).",
+		"Sets the mapping to TwinView xinerama/horizontal/vertical/leftof/aboveof/none. \n"
+		"\t\t   Values = none, vertical, horizontal, leftof, aboveof, xinerama "
+		"(default is none).",
 		XWACOM_PARAM_TWINVIEW, VALUE_OPTIONAL, RANGE, 
-		TV_NONE, TV_LEFT_RIGHT, SINGLE_VALUE, TV_NONE },
+		TV_NONE, TV_MAX, SINGLE_VALUE, TV_NONE },
 
 	{ "Mode",
 		"Switches cursor movement mode (default is absolute/on). ",
@@ -345,17 +353,30 @@ static PARAMINFO gParamInfo[] =
 		XWACOM_PARAM_TPCBUTTON, VALUE_OPTIONAL, 
 		RANGE, 0, 1, BOOLEAN_VALUE, 1 },
 
+	{ "Touch",
+		"Turns on/off Touch events (default is enable/on). ",
+		XWACOM_PARAM_TOUCH, VALUE_OPTIONAL, 
+		RANGE, 0, 1, BOOLEAN_VALUE, 1 },
+
+	{ "Capacity",
+		"Touch sensitivity level (default is 3, "
+		"-1 for none capacitive tools). ",
+		XWACOM_PARAM_CAPACITY, VALUE_OPTIONAL, 
+		RANGE, -1, 5, SINGLE_VALUE, 3 },
+
 	{ "CursorProx", 
 		"Sets cursor distance for proximity-out "
-		"in distance from the tablet.  "
-		"default is 10 for Intuos series, "
-		"42 for Graphire series).",
+		"in distance from the tablet.  \n"
+		"\t\t   (default is 10 for Intuos series, "
+		"42 for Graphire series). ",
 		XWACOM_PARAM_CURSORPROX, VALUE_OPTIONAL, RANGE, 
 		0, 255, SINGLE_VALUE, 47 },
 		
 	{ "Rotate",
 		"Sets the rotation of the tablet. "
-		"Values = NONE, CW, CCW, HALF (default is NONE).",
+		"Values = NONE, CW, CCW, HALF (default is NONE). \n"
+		"\t\t   Tablet mappings applied after this command will "
+		"be based on the new tablet orientation. ",
 		XWACOM_PARAM_ROTATE, VALUE_OPTIONAL, RANGE, 
 		ROTATE_NONE, ROTATE_HALF, SINGLE_VALUE, ROTATE_NONE },
 
@@ -426,6 +447,11 @@ static PARAMINFO gParamInfo[] =
 		XWACOM_PARAM_CLICKFORCE, VALUE_OPTIONAL, RANGE, 
 		1, 21, SINGLE_VALUE, 6 },	
 
+	{ "Threshold",
+		"Sets tip/eraser pressure threshold directly to the pressure "
+		"(default is 6*MaxZ/100)",
+		XWACOM_PARAM_THRESHOLD, VALUE_REQUIRED },	
+
 	{ "Accel",
 		"Sets relative cursor movement acceleration "
 		"(default is 1)",
@@ -584,6 +610,10 @@ static PARAMINFO gParamInfo[] =
 		"Returns the serial number of the associated device. ",
 		XWACOM_PARAM_TOOLSERIAL, VALUE_REQUIRED },
 
+	{ "Serial", 
+		"Returns/sets the serial number of the chosen device. ",
+		XWACOM_PARAM_SERIAL, VALUE_REQUIRED },
+
 	{ "TabletID", 
 		"Returns the tablet ID of the associated device. ",
 		XWACOM_PARAM_TID, VALUE_REQUIRED },
@@ -596,6 +626,10 @@ static PARAMINFO gParamInfo[] =
 		"Returns number of screens configured for the desktop. ",
 		XWACOM_PARAM_NUMSCREEN, VALUE_REQUIRED },
 
+	{ "XScaling", 
+		"Returns the status of XSCALING is set or not. ",
+		XWACOM_PARAM_XSCALING, VALUE_REQUIRED },
+
 	{ NULL }
 };
 
@@ -810,11 +844,15 @@ static int ParseValues(int nCount, const char* pszValues, int* nValues,
 		if ((p->nType == BOOLEAN_VALUE) &&
 			(!strcasecmp(pszValues,"on") ||
 			!strcasecmp(pszValues,"true") ||
+			!strcasecmp(pszValues,"enable") ||
+			!strcasecmp(pszValues,"1") ||
 			!strcasecmp(pszValues,"absolute")))
 				*nValue = 1;
 		else if ((p->nType == BOOLEAN_VALUE) &&
 			(!strcasecmp(pszValues,"off") ||
 			!strcasecmp(pszValues,"false") ||
+			!strcasecmp(pszValues,"disable") ||
+			!strcasecmp(pszValues,"0") ||
 			!strcasecmp(pszValues,"relative")))
 				*nValue = 0;
 		else if (p->nType == ACTION_VALUE)
@@ -1026,6 +1064,8 @@ static void DisplayValue (WACOMDEVICE *hDev, const char *devname, PARAMINFO *p,
 				snprintf (strval, sizeof (strval), 
 					"%s", option_char[value] );
 		}
+		else if (p->nParamID == XWACOM_PARAM_TOOLSERIAL)
+			snprintf (strval, sizeof (strval), "%u", value);
 		else
 			snprintf (strval, sizeof (strval), "%d", value);
                 break;
@@ -1132,13 +1172,14 @@ static void DisplayValue (WACOMDEVICE *hDev, const char *devname, PARAMINFO *p,
 		}
 		break;
 	default:
-		if ( ((value & AC_TYPE) != AC_KEY) &&  
-			((value & AC_TYPE) != AC_MODETOGGLE) &&
-			((value & AC_TYPE) != AC_DBLCLICK) &&
-			((value & AC_TYPE) != AC_DISPLAYTOGGLE) )
-			printf ("%d\n", value);
-		else
+		if ( (p->nType == ACTION_VALUE) && 
+			(((value & AC_TYPE) == AC_KEY) ||  
+			((value & AC_TYPE) == AC_MODETOGGLE) ||
+			((value & AC_TYPE) == AC_DBLCLICK) ||
+			((value & AC_TYPE) == AC_DISPLAYTOGGLE)) )
 			printf ("%s\n", strval);
+		else
+			printf ("%d\n", value);
 		break;
 	}
 }
