@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
+ * Copyright 2002-2009 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,6 +44,10 @@ WacomDeviceClass* wcmDeviceClasses[] =
 
 extern int xf86WcmDevSwitchModeCall(LocalDevicePtr local, int mode);
 extern void xf86WcmChangeScreen(LocalDevicePtr local, int value);
+extern void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes);
+extern void xf86WcmVirtaulTabletSize(LocalDevicePtr local);
+extern void xf86WcmVirtaulTabletPadding(LocalDevicePtr local);
+extern void xf86WcmTilt2R(WacomDeviceStatePtr ds);
 
 /*****************************************************************************
  * Static functions
@@ -55,7 +59,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 static void resetSampleCounter(const WacomChannelPtr pChannel);
 static void sendAButton(LocalDevicePtr local, int button, int mask,
 		int rx, int ry, int rz, int v3, int v4, int v5);
- 
+
 /*****************************************************************************
  * xf86WcmMappingFactor --
  *   calculate the proper tablet to screen mapping factor according to the 
@@ -65,13 +69,10 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 void xf86WcmMappingFactor(LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
-	int i = 0, minX = 0, minY = 0, maxX = 0, maxY = 0;
  
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmMappingFactor \n"));
 
-	priv->sizeX = priv->bottomX - priv->topX - 2*priv->tvoffsetX;
-	priv->sizeY = priv->bottomY - priv->topY - 2*priv->tvoffsetY;
-	priv->maxWidth = 0, priv->maxHeight = 0;
+	xf86WcmVirtaulTabletSize(local);
 	
 	if (!(priv->flags & ABSOLUTE_FLAG) || !priv->wcmMMonitor)
 	{
@@ -103,44 +104,11 @@ void xf86WcmMappingFactor(LocalDevicePtr local)
 	if (priv->currentScreen == -1) /* tool on the tablet */
 		priv->currentScreen = 0;
 
-	if ( ((priv->twinview != TV_NONE) || /* TwinView & whole desktop */
-		/* stay in one screen at a time (multimonitor) */
-		!priv->wcmMMonitor || 
-		/* always stay in the configured screen */
-		(screenInfo.numScreens > 1 && priv->screen_no != -1))  
-		 && (priv->flags & ABSOLUTE_FLAG) )
-	{
-		priv->maxWidth = priv->screenBottomX[priv->currentScreen] - 
-			priv->screenTopX[priv->currentScreen];
-		priv->maxHeight = priv->screenBottomY[priv->currentScreen] - 
-			priv->screenTopY[priv->currentScreen];
-	}
-	else
-	{
-		/* count the whole desktop when no specific screen is defined or 
-		 * tool is in relative mode
-		 */
-		minX = priv->screenTopX[0];
-		minY = priv->screenTopY[0];
-		maxX = priv->screenBottomX[0];
-		maxY = priv->screenBottomY[0];
-		for (i = 1; i < priv->numScreen; i++)
-		{
-			if (priv->screenTopX[i] < minX)
-				minX = priv->screenTopX[i];
-			if (priv->screenTopY[i] < minY)
-				minY = priv->screenTopY[i];
-			if (priv->screenBottomX[i] > maxX)
-				maxX = priv->screenBottomX[i];
-			if (priv->screenBottomY[i] > maxY)
-				maxY = priv->screenBottomY[i];
-		}
-		priv->maxWidth = maxX - minX;
-		priv->maxHeight = maxY - minY;
-	}
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmMappingFactor"
-		" Active tablet area x=%d y=%d map to maxWidth =%d maxHeight =%d\n", 
-		priv->sizeX, priv->sizeY, priv->maxWidth, priv->maxHeight));
+		" Active tablet area x=%d y=%d (virtual tablet area x=%d y=%d) map"
+		" to maxWidth =%d maxHeight =%d\n", 
+		priv->bottomX, priv->bottomY, priv->sizeX, priv->sizeY, 
+		priv->maxWidth, priv->maxHeight));
 
 	priv->factorX = (double)priv->maxWidth / (double)priv->sizeX;
 	priv->factorY = (double)priv->maxHeight / (double)priv->sizeY;
@@ -156,34 +124,59 @@ void xf86WcmMappingFactor(LocalDevicePtr local)
  *   combined horizontal and vertical setups
  ****************************************************************************/
 
-static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
+static void xf86WcmSetScreen(LocalDevicePtr local, int v0, int v1)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
-	int screenToSet = -1, letfPadding = 0, topPadding = 0;
-	int i, j, x, y, v0 = *value0, v1 = *value1;
+	int screenToSet = -1, i, j, x, y, tabletSize = 0;
 
-	DBG(6, priv->debugLevel, ErrorF("xf86WcmSetScreen "
-		"v0=%d v1=%d currentScreen=%d\n", *value0, 
-		*value1, priv->currentScreen));
+	DBG(6, priv->debugLevel, ErrorF("xf86WcmSetScreen v0=%d v1=%d "
+		"currentScreen=%d\n", v0, v1, priv->currentScreen));
+
+	if (priv->screen_no != -1 && priv->screen_no >= priv->numScreen)
+	{
+		ErrorF("xf86WcmSetScreen Screen%d is larger than number of available screens (%d)\n", 
+			priv->screen_no, priv->numScreen);
+		priv->screen_no = -1;
+	}
 
 	if (!(local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER))) return;
 
-	if (priv->twinview != TV_NONE && priv->screen_no == -1 && (priv->flags & ABSOLUTE_FLAG))
+	if (priv->twinview > TV_XINERAMA && priv->screen_no == -1 && (priv->flags & ABSOLUTE_FLAG))
 	{
 		if (priv->twinview == TV_LEFT_RIGHT)
 		{
-			if (v0 > priv->bottomX - priv->tvoffsetX && v0 <= priv->bottomX)
+			tabletSize = priv->bottomX - priv->tvoffsetX;
+			if (v0 > tabletSize && v0 <= priv->bottomX)
 				priv->currentScreen = 1;
 			if (v0 > priv->topX && v0 <= priv->topX + priv->tvoffsetX)
 				priv->currentScreen = 0;
 		}
 		if (priv->twinview == TV_ABOVE_BELOW)
 		{
-			if (v1 > priv->bottomY - priv->tvoffsetY && v1 <= priv->bottomY)
+			tabletSize = priv->bottomY - priv->tvoffsetY;
+			if (v0 > tabletSize && v0 <= priv->bottomY)
 				priv->currentScreen = 1;
-			if (v1 > priv->topY && v1 <= priv->topY + priv->tvoffsetY)
+			if (v0 > priv->topY && v0 <= priv->topY + priv->tvoffsetY)
 				priv->currentScreen = 0;
 		}
+		if (priv->twinview == TV_RIGHT_LEFT)
+		{
+			tabletSize = priv->bottomX - priv->tvoffsetX;
+			if (v0 > tabletSize && v0 <= priv->bottomX)
+				priv->currentScreen = 0;
+			if (v0 > priv->topX && v0 <= priv->topX + priv->tvoffsetX)
+				priv->currentScreen = 1;
+		}
+		if (priv->twinview == TV_BELOW_ABOVE)
+		{
+			tabletSize = priv->bottomY - priv->tvoffsetY;
+			if (v0 > tabletSize && v0 <= priv->bottomY)
+				priv->currentScreen = 0;
+			if (v0 > priv->topY && v0 <= priv->topY + priv->tvoffsetY)
+				priv->currentScreen = 1;
+		}
+		DBG(10, priv->debugLevel, ErrorF("xf86WcmSetScreen TwinView setup screenToSet=%d\n", 
+			priv->currentScreen));
 	}
 
 	xf86WcmMappingFactor(local);
@@ -218,33 +211,29 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 					break;
 			}
 		}
-		if (screenToSet != -1)
-		{
-			letfPadding = priv->screenTopX[screenToSet];
-			topPadding = priv->screenBottomX[screenToSet];
-		}
-		else
-		{
-			DBG(3, priv->debugLevel, ErrorF("xf86WcmSetScreen Error: "
-				"Can not find valid screen (currentScreen=%d)\n", 
-				priv->currentScreen));
-			return;
-		}
 	}
 	else
 		screenToSet = priv->screen_no;
 
-	x = (double)v0 * priv->factorX - letfPadding + 0.5;
-	y = (double)v1 * priv->factorY - topPadding + 0.5;
-		
+	if (screenToSet == -1)
+	{
+		DBG(3, priv->debugLevel, ErrorF("xf86WcmSetScreen Error: "
+			"Can not find valid screen (currentScreen=%d)\n", 
+			priv->currentScreen));
+		return;
+	}
+
+	xf86WcmVirtaulTabletPadding(local);
+	x = ((double)(v0 + priv->leftPadding) * priv->factorX) - priv->screenTopX[screenToSet] + 0.5;
+	y = ((double)(v1 + priv->topPadding) * priv->factorY) - priv->screenTopY[screenToSet] + 0.5;
+
 	if (x >= screenInfo.screens[screenToSet]->width)
 		x = screenInfo.screens[screenToSet]->width - 1;
 	if (y >= screenInfo.screens[screenToSet]->height)
 		y = screenInfo.screens[screenToSet]->height - 1;
 
 	xf86XInputSetScreen(local, screenToSet, x, y);
-	DBG(10, priv->debugLevel, ErrorF("xf86WcmSetScreen"
-			" current=%d ToSet=%d\n", 
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmSetScreen current=%d ToSet=%d\n", 
 			priv->currentScreen, screenToSet));
 	priv->currentScreen = screenToSet;
 }
@@ -265,7 +254,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons, int rx, int ry
 		"buttons=%d for %s\n", buttons, local->name));
 
 	/* Tablet PC buttons. */
-	if ( common->wcmTPCButton && !IsCursor(priv) && !IsPad(priv) && !IsEraser(priv) )
+	if ( common->wcmTPCButton && !IsCursor(priv) && !IsPad(priv) && !IsTouch(priv) )
 	{
 		if ( buttons & 1 )
 		{
@@ -514,21 +503,9 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 #if WCM_XINPUTABI_MAJOR == 0
 	int is_core = local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER);
 #endif
-	int button_idx, naxes = priv->naxes;
+	int naxes = priv->naxes;
 
-	if (IsPad(priv))
-		button -= 8;
-
-	if (button < 0 || button >= priv->nbuttons)
-	{
-		/* should never happen */
-		ErrorF ("sendAButton: Invalid button index %d (number of defined buttons = %d)\n", button, priv->nbuttons);
-		return; 
-	}
-
-	button_idx = button;
-	button = priv->button[button];
-	if (!button)  /* ignore this button event */
+	if (!priv->button[button])  /* ignore this button event */
 		return;
 
 #if WCM_XINPUTABI_MAJOR == 0
@@ -541,23 +518,18 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		"sendAButton TPCButton(%s) button=%d state=%d " 
 		"code=%08x, for %s coreEvent=%s \n", 
 		common->wcmTPCButton ? "on" : "off", 
-		button_idx, mask, button, 
-		local->name, (button & AC_CORE) ? "yes" : "no"));
+		button, mask, priv->button[button], 
+		local->name, (priv->button[button] & AC_CORE) ? "yes" : "no"));
 
-	switch (button & AC_TYPE)
+	switch (priv->button[button] & AC_TYPE)
 	{
 	case AC_BUTTON:
-/*		xf86PostButtonEvent(local->dev, is_absolute, button & AC_CODE,
-*/		/* Dynamically modify the button map as required --
-		 * to be moved in the place where button mappings are changed
-		 */
-		local->dev->button->map [button_idx] = button & AC_CODE;
-		xf86PostButtonEvent(local->dev, is_absolute, button_idx,
+		xf86PostButtonEvent(local->dev, is_absolute, priv->button[button] & AC_CODE,
 			mask != 0,0,naxes,rx,ry,rz,v3,v4,v5);
 		break;
 
 	case AC_KEY:
-		sendKeystroke(local, button, priv->keys[button_idx], mask);
+		sendKeystroke(local, priv->button[button], priv->keys[button], mask);
 		break;
 
 	case AC_MODETOGGLE:
@@ -598,27 +570,18 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		break;
 
 	case AC_DBLCLICK:
-		/* Dynamically modify the button map as required --
-		 * to be moved in the place where button mappings are changed.
-		 * Only left double is supported.
-		 */
-		local->dev->button->map [button_idx] = 1;
-
 		if (mask)
 		{
 			/* Left button down */
 			xf86PostButtonEvent(local->dev, is_absolute,
-				button_idx, 1,0,naxes, 
-				rx,ry,rz,v3,v4,v5);
-
+				1,1,0,naxes, rx,ry,rz,v3,v4,v5);
 			/* Left button up */
 			xf86PostButtonEvent(local->dev, is_absolute,
-				button_idx,0,0,naxes,
-				rx,ry,rz,v3,v4,v5);
+				1,0,0,naxes,rx,ry,rz,v3,v4,v5);
 		}
 
 		/* Left button down/up upon mask is 1/0 */
-		xf86PostButtonEvent(local->dev, is_absolute, button_idx, 
+		xf86PostButtonEvent(local->dev, is_absolute, 1, 
 			mask != 0,0,naxes,rx,ry,rz,v3,v4,v5);
 		break;
 	}
@@ -635,7 +598,8 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
  *   Send events defined for relative/absolute wheels or strips
  ****************************************************************************/
 
-static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* ds, int x, int y, int z, int v3, int v4, int v5)
+static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* ds, 
+		int x, int y, int z, int v3, int v4, int v5)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	int fakeButton = 0, i, value = 0, naxes = priv->naxes;
@@ -746,13 +710,11 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 	switch (fakeButton & AC_TYPE)
 	{
 	    case AC_BUTTON:
-		/* send both button on/off in the same event for pad */
-		local->dev->button->map [0] = fakeButton & AC_CODE;
-	
-		xf86PostButtonEvent(local->dev, is_absolute, 0,
+		/* send both button on/off in the same event for pad */	
+		xf86PostButtonEvent(local->dev, is_absolute, fakeButton & AC_CODE,
 			1,0,naxes,x,y,z,v3,v4,v5);
 
-		xf86PostButtonEvent(local->dev, is_absolute, 0,
+		xf86PostButtonEvent(local->dev, is_absolute, fakeButton & AC_CODE,
 			0,0,naxes,x,y,z,v3,v4,v5);
 	    break;
 
@@ -782,14 +744,9 @@ static void sendCommonEvents(LocalDevicePtr local, const WacomDeviceState* ds, i
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	int buttons = ds->buttons;
-	int naxes = priv->naxes;
 
-	/* don't emit proximity events if device does not support proximity */
-	if ((local->dev->proximity && !priv->oldProximity))
-		xf86PostProximityEvent(local->dev, 1, 0, naxes,
-			x, y, z, v3, v4, v5);
-
-	if (priv->oldButtons != buttons)
+	/* send button events when state changed or first time in prox and button unpresses */
+	if (priv->oldButtons != buttons || (!priv->oldProximity && !buttons))
 		xf86WcmSendButtons(local,buttons,x,y,z,v3,v4,v5);
 
 	/* emulate wheel/strip events when defined */
@@ -825,7 +782,6 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 	WacomCommonPtr common = priv->common;
 	int naxes = priv->naxes;
 	int is_absolute = priv->flags & ABSOLUTE_FLAG;
-
 	int v3, v4, v5;
 	int no_jitter; 
 	double relacc, param;
@@ -833,7 +789,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 	if (priv->serial && serial != priv->serial)
 	{
 		DBG(10, priv->debugLevel, ErrorF("[%s] serial number"
-			" is %d but your system configured %d", 
+			" is %u but your system configured %u", 
 			local->name, serial, (int)priv->serial));
 		return;
 	}
@@ -879,7 +835,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		v3 = rot;
 		v4 = throttle;
 	}
-	else
+	else  /* Intuos styli have tilt */
 	{
 		v3 = tx;
 		v4 = ty;
@@ -888,7 +844,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 
 	DBG(6, priv->debugLevel, ErrorF("[%s] %s prox=%d\tx=%d"
 		"\ty=%d\tz=%d\tv3=%d\tv4=%d\tv5=%d\tid=%d"
-		"\tserial=%d\tbutton=%s\tbuttons=%d\n",
+		"\tserial=%u\tbutton=%s\tbuttons=%d\n",
 		local->name,
 		is_absolute ? "abs" : "rel",
 		is_proximity,
@@ -907,10 +863,12 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		priv->oldZ = z;
 		priv->oldTiltX = tx;
 		priv->oldTiltY = ty;
+		priv->oldCapacity = ds->capacity;
 		priv->oldStripX = ds->stripx;
 		priv->oldStripY = ds->stripy;
 		priv->oldRot = rot;
 		priv->oldThrottle = throttle;
+		priv->oldButtons = 0;
 	}
 	if (!is_absolute)
 	{
@@ -955,7 +913,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			 * screen and modify the axes before posting events */
 			if(!(priv->flags & BUTTONS_ONLY_FLAG))
 			{
-				xf86WcmSetScreen(local, &x, &y);
+				xf86WcmSetScreen(local, x, y);
 			}
 
 			/* unify acceleration in both directions 
@@ -963,6 +921,13 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			 */
 			if (!is_absolute)
 				x *= priv->factorY / priv->factorX;
+ 			else
+			{
+				/* Padding virtual values */
+				xf86WcmVirtaulTabletPadding(local);
+				x += priv->leftPadding;
+				y += priv->topPadding;
+			}
 
 #ifdef WCM_XORG_TABLET_SCALING
 			/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
@@ -970,20 +935,41 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			/* The +-0.4 is to increase the sensitivity in relative mode.
 			 * Must be sensitive to which way the tool is moved or one way
 			 * will get a severe penalty for small movements. */
-			if(is_absolute) {
+ 			if(is_absolute) {
 				x -= priv->topX;
 				y -= priv->topY;
+				if (priv->currentScreen == 1 && priv->twinview > TV_XINERAMA)
+				{
+					x -= priv->tvoffsetX;
+					y -= priv->tvoffsetY;
+				}
 			}
 			x = (int)((double)x * priv->factorX + (x>=0?0.4:-0.4));
 			y = (int)((double)y * priv->factorY + (y>=0?0.4:-0.4));
 
-			/* map to a specific screen */
-			if (priv->screen_no != -1 || priv->twinview != TV_NONE)
+			if ((priv->flags & ABSOLUTE_FLAG) && (priv->twinview <= TV_XINERAMA))
 			{
-				x += priv->screenTopX[priv->currentScreen];
-				y += priv->screenTopY[priv->currentScreen];
+				x -= priv->screenTopX[priv->currentScreen];
+				y -= priv->screenTopY[priv->currentScreen];
 			}
+
+			if (priv->screen_no != -1)
+			{
+				if (x > priv->screenBottomX[priv->currentScreen] - priv->screenTopX[priv->currentScreen])
+					x = priv->screenBottomX[priv->currentScreen];
+				if (x < 0) x = 0;
+				if (y > priv->screenBottomY[priv->currentScreen] - priv->screenTopY[priv->currentScreen])
+					y = priv->screenBottomY[priv->currentScreen];
+				if (y < 0) y = 0;
+	
+			}
+			priv->currentSX = x;
+			priv->currentSY = y;
 #endif
+			/* don't emit proximity events if device does not support proximity */
+			if ((local->dev->proximity && !priv->oldProximity))
+				xf86PostProximityEvent(local->dev, 1, 0, naxes, x, y, z, v3, v4, v5);
+
 			/* Move the cursor to where it should be before sending button events */
 			if(!(priv->flags & BUTTONS_ONLY_FLAG))
 				xf86PostMotionEvent(local->dev, is_absolute,
@@ -997,6 +983,13 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		{
 			buttons = 0;
 
+#ifdef WCM_XORG_TABLET_SCALING
+			/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
+			 * for coordinate conversion at the moment */
+			x = priv->currentSX;
+			y = priv->currentSY;
+#endif
+
 			/* reports button up when the device has been
 			 * down and becomes out of proximity */
 			if (priv->oldButtons)
@@ -1008,12 +1001,18 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 	}
 	else
 	{
+
 		if (v3 || v4 || v5 || buttons || ds->relwheel)
 		{
 			x = 0;
 			y = 0;
 			if ( v3 || v4 || v5 )
-				xf86WcmSetScreen(local, &x, &y);
+				xf86WcmSetScreen(local, x, y);
+
+			/* don't emit proximity events if device does not support proximity */
+			if ((local->dev->proximity && !priv->oldProximity))
+			xf86PostProximityEvent(local->dev, 1, 0, naxes, x, y, z, v3, v4, v5);
+
 			sendCommonEvents(local, ds, x, y, z, v3, v4, v5);
 			is_proximity = 1;
 			/* xf86PostMotionEvent is only needed to post the valuators
@@ -1046,6 +1045,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		priv->oldX = priv->currentX;
 		priv->oldY = priv->currentY;
 		priv->oldZ = z;
+		priv->oldCapacity = ds->capacity;
 		priv->oldTiltX = tx;
 		priv->oldTiltY = ty;
 		priv->oldStripX = ds->stripx;
@@ -1060,6 +1060,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		priv->oldX = 0;
 		priv->oldY = 0;
 		priv->oldZ = 0;
+		priv->oldCapacity = ds->capacity;
 		priv->oldTiltX = 0;
 		priv->oldTiltY = 0;
 		priv->oldStripX = 0;
@@ -1090,6 +1091,7 @@ static int xf86WcmSuppress(WacomCommonPtr common, const WacomDeviceState* dsOrig
 	if (ABS(dsOrig->tiltx - dsNew->tiltx) > suppress) returnV = 1;
 	if (ABS(dsOrig->tilty - dsNew->tilty) > suppress) returnV = 1;
 	if (ABS(dsOrig->pressure - dsNew->pressure) > suppress) returnV = 1;
+	if (ABS(dsOrig->capacity - dsNew->capacity) > suppress) returnV = 1;
 	if (ABS(dsOrig->throttle - dsNew->throttle) > suppress) returnV = 1;
 	if (ABS(dsOrig->rotation - dsNew->rotation) > suppress &&
 		(1800 - ABS(dsOrig->rotation - dsNew->rotation)) >  suppress) returnV = 1;
@@ -1100,11 +1102,14 @@ static int xf86WcmSuppress(WacomCommonPtr common, const WacomDeviceState* dsOrig
 	if ((ABS(dsOrig->abswheel - dsNew->abswheel) > suppress) 
 		|| (dsNew->relwheel != 0)) returnV = 1;
 
-	/* need to check if cursor moves or not */
+	/* cursor moves or not? */
 	if ((ABS(dsOrig->x - dsNew->x) > suppress) || 
 			(ABS(dsOrig->y - dsNew->y) > suppress)) 
-		returnV = 2;
-	else if (returnV == 1) /* don't move the cursor */
+	{
+		if (!returnV) /* need to check if cursor moves or not */
+			returnV = 2;
+	}
+	else /* don't move cursor */
 	{
 		dsNew->x = dsOrig->x;
 		dsNew->y = dsOrig->y;
@@ -1216,7 +1221,8 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 
 #ifdef WCM_ENABLE_LINUXINPUT
 	/* Discard the first 2 USB packages due to events delay */
-	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) && ds.device_type != PAD_ID )
+	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) && 
+		(ds.device_type != PAD_ID) )
 	{
 		DBG(11, common->debugLevel, 
 			ErrorF("discarded %dth USB data.\n", 
@@ -1225,6 +1231,13 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		return; /* discard */
 	}
 #endif
+
+	if (strstr(common->wcmModel->name, "Intuos4"))
+	{
+		/* convert Intuos4 mouse tilt to rotation */
+		xf86WcmTilt2R(&ds);
+	}
+
 	fs = &pChannel->rawFilter;
 	if (!fs->npoints && ds.proximity)
 	{
@@ -1247,7 +1260,7 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		}
 		fs->x[0] = ds.x;
 		fs->y[0] = ds.y;
-		if (HANDLE_TILT(common) && (ds.device_type == STYLUS_ID || ds.device_type == ERASER_ID))
+		if (HANDLE_TILT(common) && ((ds.device_type == STYLUS_ID) || (ds.device_type == ERASER_ID)))
 		{
 			for (i=common->wcmRawSample - 1; i>0; i--)
 			{
@@ -1257,7 +1270,8 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 			fs->tiltx[0] = ds.tiltx;
 			fs->tilty[0] = ds.tilty;
 		}
-		if (RAW_FILTERING(common) && common->wcmModel->FilterRaw && ds.device_type != PAD_ID)
+
+		if (RAW_FILTERING(common) && common->wcmModel->FilterRaw && (ds.device_type != PAD_ID))
 		{
 			if (common->wcmModel->FilterRaw(common,pChannel,&ds))
 			{
@@ -1267,7 +1281,6 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 				return; /* discard */
 			}
 		}
-
 		/* Discard unwanted data */
 		suppress = xf86WcmSuppress(common, pLast, &ds);
 		if (!suppress)
@@ -1288,8 +1301,8 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		pChannel->valid.states,
 		sizeof(WacomDeviceState) * (common->wcmRawSample - 1));
 	pChannel->valid.state = ds; /*save last raw sample */
-	if (pChannel->nSamples < common->wcmRawSample) ++pChannel->nSamples;
 
+	if (pChannel->nSamples < common->wcmRawSample) ++pChannel->nSamples;
 	commonDispatchDevice(common,channel,pChannel, suppress);
 	resetSampleCounter(pChannel);
 }
@@ -1372,7 +1385,6 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 					break;
 				}
 	}
-
 	DBG(10, common->debugLevel, ErrorF("commonDispatchDevice device type = %d\n", ds->device_type));
 	/* Find the device the current events are meant for */
 	/* 1: Find the tool (the one with correct serial or in second
@@ -1435,6 +1447,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 				/* Send soft prox-out for the old area */
 				LocalDevicePtr oDev = outprox->device;
 				WacomDeviceState out = { 0 };
+				out.device_type = DEVICE_ID(((WacomDevicePtr)(oDev->private))->flags);
 				DBG(2, common->debugLevel, ErrorF("Soft prox-out for %s\n",
 					outprox->device->name));
 				xf86WcmSendEvents(oDev, &out);
@@ -1461,10 +1474,8 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	if (pDev && !miPointerGetScreen(pDev->dev))
 #endif
 	{
-		DBG(1, common->debugLevel, ErrorF("xf86WcmEvent: "
-			"Wacom driver can not get Current Screen ID\n"));
-		DBG(1, common->debugLevel, ErrorF(
-			"Please remove Wacom tool from the tablet.\n"));
+		ErrorF("xf86WcmEvent: Wacom driver can not get Current Screen ID\n");
+		ErrorF("Please remove Wacom tool from the tablet and bring it back again.\n");
 		return;
 	}
 
@@ -1478,17 +1489,77 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 		/* button 1 Threshold test */
 		int button = 1;
 		priv = pDev->private;
-		if ( IsStylus(priv) || IsEraser(priv))
+
+#ifdef WCM_ENABLE_LINUXINPUT
+		if (common->wcmDevCls == &gWacomUSBDevice && IsTouch(priv) && !ds->proximity)
 		{
-			if (filtered.pressure < common->wcmThreshold )
-				filtered.buttons &= ~button;
-			else
+			priv->hardProx = 0;
+		}		
+
+		if (common->wcmDevCls == &gWacomUSBDevice && (IsStylus(priv) || IsEraser(priv)))
+		{
+			priv->hardProx = 1;
+		}
+		
+		/* send a touch out for USB Tablet PCs */
+		if (common->wcmDevCls == &gWacomUSBDevice && !IsTouch(priv) 
+			&& common->wcmTouchDefault && !priv->oldProximity)
+		{ 
+			LocalDevicePtr localDevices = xf86FirstLocalDevice();
+			WacomCommonPtr tempcommon = NULL;
+			WacomDevicePtr temppriv = NULL;
+
+			/* Lookup to see if associated touch was enabled */
+			for (; localDevices != NULL; localDevices = localDevices->next)
+			{
+				if (strstr(localDevices->drv->driverName, "wacom"))
+				{
+					temppriv = (WacomDevicePtr) localDevices->private;
+					tempcommon = temppriv->common;
+
+					if ((tempcommon->tablet_id == common->tablet_id) && 
+						IsTouch(temppriv) && temppriv->oldProximity)
+					{
+						/* Send soft prox-out for touch first */
+						WacomDeviceState out = { 0 };
+						out.device_type = DEVICE_ID(temppriv->flags);
+						DBG(2, common->debugLevel, ErrorF(
+							"Send soft prox-out for %s first\n",
+							localDevices->name));
+						xf86WcmSendEvents(localDevices, &out);
+					}
+				}
+			}
+		}
+#endif /* WCM_ENABLE_LINUXINPUT */
+
+		if (IsStylus(priv) || IsEraser(priv))
+		{
+			/* set button1 (left click) on/off */
+			if (filtered.pressure >= common->wcmThreshold)
 				filtered.buttons |= button;
+			else
+			{
+				/* threshold tolerance */
+				int tol = common->wcmMaxZ / 250;
+				if (strstr(common->wcmModel->name, "Intuos4"))
+					tol = common->wcmMaxZ / 125;
+				if (filtered.pressure < common->wcmThreshold - tol)
+					filtered.buttons &= ~button;
+			}
 			/* transform pressure */
 			transPressureCurve(priv,&filtered);
 		}
 
-		if (!(priv->flags & ABSOLUTE_FLAG) && !priv->hardProx)
+		/* touch capacity is supported */
+		if (IsTouch(priv) && (common->wcmCapacityDefault >= 0) && !priv->hardProx)
+		{
+			if (((double)(filtered.capacity * 5) / 
+					(double)common->wcmMaxZ) > 
+					(5 - common->wcmCapacity))
+				filtered.buttons |= button;
+		}
+		else if (IsCursor(priv) && !priv->hardProx)
 		{
 			/* initial current max distance */
 			if (strstr(common->wcmModel->name, "Intuos"))
@@ -1496,8 +1567,10 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 			else
 				common->wcmMaxCursorDist = 0;
 		}
+
 		/* Store current hard prox for next use */
-		priv->hardProx = ds->proximity;		
+		if (!IsTouch(priv))
+			priv->hardProx = ds->proximity;		
 
 		/* User-requested filtering comes next */
 
@@ -1547,21 +1620,31 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 		#endif /* throttle */
 
-		if (!(priv->flags & ABSOLUTE_FLAG) && (suppress == 2))
+		if (!(priv->flags & ABSOLUTE_FLAG) && !IsPad(priv))
 		{
 			/* To improve the accuracy of relative x/y,
-			 * don't send event when there is no movement.
+			 * don't send motion event when there is no movement.
 			 */
 			double deltx = filtered.x - priv->oldX;
 			double delty = filtered.y - priv->oldY;
 			deltx *= priv->factorY*priv->speed;
 			delty *= priv->factorY*priv->speed;
-
+	
 			if (ABS(deltx)<1 && ABS(delty)<1) 
 			{
-				DBG(10, common->debugLevel, ErrorF(
-					"Ignore non-movement relative data \n"));
-				return;
+				/* don't move the cursor */
+				if (suppress == 1) 
+				{
+					/* send other events, such as button/wheel */
+					filtered.x = priv->oldX;
+					filtered.y = priv->oldY;
+				}
+				else /* no other events to send */
+				{
+					DBG(10, common->debugLevel, ErrorF(
+						"Ignore non-movement relative data \n"));
+					return;
+				}
 			}
 			else
 			{
@@ -1574,10 +1657,11 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 			}
 		}
 
-		/* force out-prox when distance is outside wcmCursorProxoutDist. */
-		if (!(priv->flags & ABSOLUTE_FLAG))
+		/* force out-prox when distance is outside wcmCursorProxoutDist for pucks */
+		if (IsCursor(priv))
 		{
-			if (strstr(common->wcmModel->name, "Intuos"))
+			/* force out-prox when distance is outside wcmCursorProxoutDist. */
+			if (common->wcmProtocolLevel == 5)
 			{
 				if (common->wcmMaxCursorDist > filtered.distance)
 					common->wcmMaxCursorDist = filtered.distance;
@@ -1597,7 +1681,8 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 			if (priv->oldProximity)
 			{
-				if (abs(filtered.distance - common->wcmMaxCursorDist) > common->wcmCursorProxoutDist)
+				if (abs(filtered.distance - common->wcmMaxCursorDist) 
+						> common->wcmCursorProxoutDist)
 					filtered.proximity = 0;
 			}
 			/* once it is out. Don't let it in until a hard in */
@@ -1621,7 +1706,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	else
 	{
 		DBG(11, common->debugLevel, ErrorF("no device matches with"
-				" id=%d, serial=%d\n",
+				" id=%d, serial=%u\n",
 				ds->device_type, ds->serial_num));
 	}
 }
@@ -1651,7 +1736,10 @@ int xf86WcmInitTablet(LocalDevicePtr local, const char* id, float version)
 	if (common->wcmThreshold <= 0)
 	{
 		/* Threshold for counting pressure as a button */
-		common->wcmThreshold = common->wcmMaxZ * 3 / 50;
+		if (strstr(common->wcmModel->name, "Intuos4"))
+			common->wcmThreshold = common->wcmMaxZ * 3 / 25;
+		else
+			common->wcmThreshold = common->wcmMaxZ * 3 / 50;
 		ErrorF("%s Wacom using pressure threshold of %d for button 1\n",
 			XCONFIG_PROBED, common->wcmThreshold);
 	}
@@ -1694,10 +1782,10 @@ int xf86WcmInitTablet(LocalDevicePtr local, const char* id, float version)
 
 	/* output tablet state as probed */
 	if (xf86Verbose)
-		ErrorF("%s Wacom %s tablet speed=%d maxX=%d maxY=%d maxZ=%d "
+		ErrorF("%s Wacom %s tablet speed=%d (%d) maxX=%d maxY=%d maxZ=%d "
 			"resX=%d resY=%d  tilt=%s\n",
 			XCONFIG_PROBED,
-			model->name, common->wcmLinkSpeed,
+			model->name, common->wcmLinkSpeed, common->wcmISDV4Speed, 
 			common->wcmMaxX, common->wcmMaxY, common->wcmMaxZ,
 			common->wcmResolX, common->wcmResolY,
 			HANDLE_TILT(common) ? "enabled" : "disabled");
@@ -1743,12 +1831,12 @@ static void xf86WcmInitialTVScreens(LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 
-	if (priv->twinview == TV_NONE)
+	if (priv->twinview <= TV_XINERAMA)
 		return;
 
 	priv->numScreen = 2;
 
-	if (priv->twinview == TV_LEFT_RIGHT)
+	if ((priv->twinview == TV_LEFT_RIGHT) || (priv->twinview == TV_RIGHT_LEFT))
 	{
 		/* it does not need the offset if always map to a specific screen */
 		if (priv->screen_no == -1)
@@ -1766,7 +1854,7 @@ static void xf86WcmInitialTVScreens(LocalDevicePtr local)
 			priv->tvResolution[3] = priv->tvResolution[1];
 		}
 	}
-	else if (priv->twinview == TV_ABOVE_BELOW)
+	else if ((priv->twinview == TV_ABOVE_BELOW) || (priv->twinview == TV_BELOW_ABOVE))
 	{
 		/* it does not need the offset if always map to a specific screen */
 		if (priv->screen_no == -1)
@@ -1786,12 +1874,12 @@ static void xf86WcmInitialTVScreens(LocalDevicePtr local)
 	}
 
 	/* initial screen info */
-	priv->screenTopX[0] = 0;
-	priv->screenTopY[0] = 0;
-	priv->screenBottomX[0] = priv->tvResolution[0];
-	priv->screenBottomY[0] = priv->tvResolution[1];
 	if (priv->twinview == TV_ABOVE_BELOW)
 	{
+		priv->screenTopX[0] = 0;
+		priv->screenTopY[0] = 0;
+		priv->screenBottomX[0] = priv->tvResolution[0];
+		priv->screenBottomY[0] = priv->tvResolution[1];
 		priv->screenTopX[1] = 0;
 		priv->screenTopY[1] = priv->tvResolution[1];
 		priv->screenBottomX[1] = priv->tvResolution[2];
@@ -1799,11 +1887,37 @@ static void xf86WcmInitialTVScreens(LocalDevicePtr local)
 	}
 	if (priv->twinview == TV_LEFT_RIGHT)
 	{
+		priv->screenTopX[0] = 0;
+		priv->screenTopY[0] = 0;
+		priv->screenBottomX[0] = priv->tvResolution[0];
+		priv->screenBottomY[0] = priv->tvResolution[1];
 		priv->screenTopX[1] = priv->tvResolution[0];
 		priv->screenTopY[1] = 0;
 		priv->screenBottomX[1] = priv->tvResolution[0] + priv->tvResolution[2];
 		priv->screenBottomY[1] = priv->tvResolution[3];
 	}
+	if (priv->twinview == TV_BELOW_ABOVE)
+	{
+		priv->screenTopX[0] = 0;
+		priv->screenTopY[0] = priv->tvResolution[1];
+		priv->screenBottomX[0] = priv->tvResolution[2];
+		priv->screenBottomY[0] = priv->tvResolution[1] + priv->tvResolution[3];
+		priv->screenTopX[1] = 0;
+		priv->screenTopY[1] = 0;
+		priv->screenBottomX[1] = priv->tvResolution[0];
+		priv->screenBottomY[1] = priv->tvResolution[1];
+	}
+	if (priv->twinview == TV_RIGHT_LEFT)
+	{
+		priv->screenTopX[0] = priv->tvResolution[0];
+		priv->screenTopY[0] = 0;
+		priv->screenBottomX[0] = priv->tvResolution[0] + priv->tvResolution[2];
+		priv->screenBottomY[0] = priv->tvResolution[3];
+		priv->screenTopX[1] = 0;
+		priv->screenTopY[1] = 0;
+		priv->screenBottomX[1] = priv->tvResolution[0];
+		priv->screenBottomY[1] = priv->tvResolution[1];
+	}
 
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialTVScreens for \"%s\" "
 		"topX0=%d topY0=%d bottomX0=%d bottomY0=%d "
@@ -1823,9 +1937,11 @@ void xf86WcmInitialScreens(LocalDevicePtr local)
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	int i;
 
+	DBG(2, priv->debugLevel, ErrorF("xf86WcmInitialScreens for \"%s\" "
+		"number of screen=%d \n", local->name, screenInfo.numScreens));
 	priv->tvoffsetX = 0;
 	priv->tvoffsetY = 0;
-	if (priv->twinview != TV_NONE)
+	if (priv->twinview > TV_XINERAMA)
 	{
 		xf86WcmInitialTVScreens(local);
 		return;
@@ -1840,14 +1956,17 @@ void xf86WcmInitialScreens(LocalDevicePtr local)
 	for (i=0; i<screenInfo.numScreens; i++)
 	{
 #ifdef WCM_HAVE_DIXSCREENORIGINS
-		priv->screenTopX[i] = dixScreenOrigins[i].x;
-		priv->screenTopY[i] = dixScreenOrigins[i].y;
-		priv->screenBottomX[i] = dixScreenOrigins[i].x;
-		priv->screenBottomY[i] = dixScreenOrigins[i].y;
+		if (screenInfo.numScreens > 1)
+		{
+			priv->screenTopX[i] = dixScreenOrigins[i].x;
+			priv->screenTopY[i] = dixScreenOrigins[i].y;
+			priv->screenBottomX[i] = dixScreenOrigins[i].x;
+			priv->screenBottomY[i] = dixScreenOrigins[i].y;
 
-		DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialScreens from dix for \"%s\" "
-			"ScreenOrigins[%d].x=%d ScreenOrigins[%d].y=%d \n",
-			local->name, i, priv->screenTopX[i], i, priv->screenTopY[i]));
+			DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialScreens from dix for \"%s\" "
+				"ScreenOrigins[%d].x=%d ScreenOrigins[%d].y=%d \n",
+				local->name, i, priv->screenTopX[i], i, priv->screenTopY[i]));
+		}
 #else /* WCM_HAVE_DIXSCREENORIGINS */
 		if (i > 0)
 		{
@@ -1867,3 +1986,146 @@ void xf86WcmInitialScreens(LocalDevicePtr local)
 			i, priv->screenBottomX[i], i, priv->screenBottomY[i]));
 	}
 }
+
+/*****************************************************************************
+ * rotateOneTool
+ ****************************************************************************/
+
+static void rotateOneTool(WacomDevicePtr priv)
+{
+	WacomCommonPtr common = priv->common;
+	WacomToolAreaPtr area = priv->toolarea;
+	int tmpTopX, tmpTopY, tmpBottomX, tmpBottomY, oldMaxX, oldMaxY;
+
+	DBG(10, priv->debugLevel, ErrorF("rotateOneTool for \"%s\" \n", priv->local->name));
+
+	oldMaxX = priv->wcmMaxX;
+	oldMaxY = priv->wcmMaxY;
+
+	tmpTopX = priv->topX;
+	tmpBottomX = priv->bottomX;
+	tmpTopY = priv->topY;
+	tmpBottomY = priv->bottomY;
+
+	if (common->wcmRotate == ROTATE_CW || common->wcmRotate == ROTATE_CCW)
+	{
+	    priv->wcmMaxX = oldMaxY;
+	    priv->wcmMaxY = oldMaxX;
+	}
+
+	switch (common->wcmRotate) {
+	      case ROTATE_CW:
+		area->topX = priv->topX = tmpTopY;
+		area->bottomX = priv->bottomX = tmpBottomY;
+		area->topY = priv->topY = oldMaxX - tmpBottomX;
+		area->bottomY = priv->bottomY =oldMaxX - tmpTopX;
+		break;
+	      case ROTATE_CCW:
+		area->topX = priv->topX = oldMaxY - tmpBottomY;
+		area->bottomX = priv->bottomX = oldMaxY - tmpTopY;
+		area->topY = priv->topY = tmpTopX;
+		area->bottomY = priv->bottomY = tmpBottomX;
+		break;
+	      case ROTATE_HALF:
+		area->topX = priv->topX = oldMaxX - tmpBottomX;
+		area->bottomX = priv->bottomX = oldMaxX - tmpTopX;
+		area->topY = priv->topY= oldMaxY - tmpBottomY;
+		area->bottomY = priv->bottomY = oldMaxY - tmpTopY;
+		break;
+	}
+	xf86WcmMappingFactor(priv->local);
+	xf86WcmInitialCoordinates(priv->local, 0);
+	xf86WcmInitialCoordinates(priv->local, 1);
+
+	if (tmpTopX != priv->topX)
+		xf86ReplaceIntOption(priv->local->options, "TopX", priv->topX);
+	if (tmpTopY != priv->topY)
+		xf86ReplaceIntOption(priv->local->options, "TopY", priv->topY);
+	if (tmpBottomX != priv->bottomX)
+		xf86ReplaceIntOption(priv->local->options, "BottomX", priv->bottomX);
+	if (tmpBottomY != priv->bottomY)
+		xf86ReplaceIntOption(priv->local->options, "BottomY", priv->bottomY);
+}
+
+/*****************************************************************************
+ * xf86WcmRotateTablet
+ ****************************************************************************/
+
+void xf86WcmRotateTablet(LocalDevicePtr local, int value)
+{
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
+	WacomDevicePtr tmppriv;
+	int oldRotation;
+	int tmpTopX, tmpTopY, tmpBottomX, tmpBottomY, oldMaxX, oldMaxY;
+
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmRotateTablet for \"%s\" \n", local->name));
+
+	if (common->wcmRotate == value) /* initialization */
+	{
+		rotateOneTool(priv);
+	}
+	else
+	{
+		oldRotation = common->wcmRotate;
+		common->wcmRotate = value;
+
+		/* rotate all devices at once! else they get misaligned */
+		for (tmppriv = common->wcmDevices; tmppriv; tmppriv = tmppriv->next)
+		{
+		    oldMaxX = tmppriv->wcmMaxX;
+		    oldMaxY = tmppriv->wcmMaxY;
+
+		    if (oldRotation == ROTATE_CW || oldRotation == ROTATE_CCW) 
+		    {
+			tmppriv->wcmMaxX = oldMaxY;
+			tmppriv->wcmMaxY = oldMaxX;
+		    }
+
+		    tmpTopX = tmppriv->topX;
+		    tmpBottomX = tmppriv->bottomX;
+		    tmpTopY = tmppriv->topY;
+		    tmpBottomY = tmppriv->bottomY;
+
+		    /* recover to the unrotated xy-rectangles */
+		    switch (oldRotation) {
+		      case ROTATE_CW:
+			tmppriv->topX = oldMaxY - tmpBottomY;
+			tmppriv->bottomX = oldMaxY - tmpTopY;
+			tmppriv->topY = tmpTopX;
+			tmppriv->bottomY = tmpBottomX;
+			break;
+		      case ROTATE_CCW:
+			tmppriv->topX = tmpTopY;
+			tmppriv->bottomX = tmpBottomY;
+			tmppriv->topY = oldMaxX - tmpBottomX;
+			tmppriv->bottomY = oldMaxX - tmpTopX;
+			break;
+		      case ROTATE_HALF:
+			tmppriv->topX = oldMaxX - tmpBottomX;
+			tmppriv->bottomX = oldMaxX - tmpTopX;
+			tmppriv->topY = oldMaxY - tmpBottomY;
+			tmppriv->bottomY = oldMaxY - tmpTopY;
+			break;
+		    }
+
+		    /* and rotate them to the new value */
+		    rotateOneTool(tmppriv);
+
+		    switch(value) {
+			case ROTATE_NONE:
+			    xf86ReplaceStrOption(local->options, "Rotate", "NONE");
+			break;
+			case ROTATE_CW:
+			    xf86ReplaceStrOption(local->options, "Rotate", "CW");
+			break;
+			case ROTATE_CCW:
+			    xf86ReplaceStrOption(local->options, "Rotate", "CCW");
+			break;
+			case ROTATE_HALF:
+			    xf86ReplaceStrOption(local->options, "Rotate", "HALF");
+			break;
+		    }
+		}
+	}
+}
