@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2008 by Ping Cheng, Wacom. <pingc@wacom.com>
+ * Copyright 2002-2009 by Ping Cheng, Wacom. <pingc@wacom.com>
  *                                                                            
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -137,19 +137,8 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 
 	/* Default button and expresskey values */
 	for (i=0; i<MAX_BUTTONS; i++)
-		priv->button[i] = IsPad (priv) ?
-			(AC_BUTTON | (MAX_MOUSE_BUTTONS/2 + i + 1)) : (AC_BUTTON | (i + 1));
-/*	for (i=MAX_MOUSE_BUTTONS/2; i<MAX_BUTTONS; i++)
-		priv->button[i] = IsPad (priv) ?
-			(AC_KEY | (XK_F1 + i)) : (AC_BUTTON | (i + 1));
-*/
-	/* Now for backward compatibility make some keys emit button events
-	 * with button indices 9-16...
-	 */
-/*	if (IsPad (priv))
-		for (i = 0; i < 8; i++)
-			priv->button[i] = (AC_BUTTON | (i + 9));
-*/
+		priv->button[i] = (AC_BUTTON | (i + 1));
+
 	for (i=0; i<MAX_BUTTONS; i++)
 		for (j=0; j<256; j++)
 			priv->keys[i][j] = 0;
@@ -158,8 +147,8 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	priv->relup = 5;			/* Default relative wheel up event */
 	priv->reldn = 4;			/* Default relative wheel down event */
 	
-	priv->wheelup = IsPad (priv) ? 5 : 0;	/* Default absolute wheel up event */
-	priv->wheeldn = IsPad (priv) ? 4 : 0;	/* Default absolute wheel down event */
+	priv->wheelup = IsPad (priv) ? 4 : 0;	/* Default absolute wheel up event */
+	priv->wheeldn = IsPad (priv) ? 5 : 0;	/* Default absolute wheel down event */
 	priv->striplup = 4;			/* Default left strip up event */
 	priv->stripldn = 5;			/* Default left strip down event */
 	priv->striprup = 4;			/* Default right strip up event */
@@ -171,12 +160,14 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 
 	priv->maxWidth = 0;			/* max active screen width */
 	priv->maxHeight = 0;			/* max active screen height */
+	priv->leftPadding = 0;			/* left padding for virtual tablet */
+	priv->topPadding = 0;			/* top padding for virtual tablet */
 	priv->twinview = TV_NONE;		/* not using twinview gfx */
 	priv->tvoffsetX = 0;			/* none X edge offset for TwinView setup */
 	priv->tvoffsetY = 0;			/* none Y edge offset for TwinView setup */
 	for (i=0; i<4; i++)
 		priv->tvResolution[i] = 0;	/* unconfigured twinview resolution */
-	priv->wcmMMonitor = 1;		/* enabled (=1) to support multi-monitor desktop. */
+	priv->wcmMMonitor = 1;			/* enabled (=1) to support multi-monitor desktop. */
 						/* disabled (=0) when user doesn't want to move the */
 						/* cursor from one screen to another screen */
 
@@ -198,16 +189,22 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 
 	common->wcmDevCls = &gWacomSerialDevice; /* device-specific functions */
 	common->wcmModel = NULL;                 /* model-specific functions */
-	common->wcmEraserID = 0;	/* eraser id associated with the stylus */
+	common->wcmEraserID = 0;	 /* eraser id associated with the stylus */
 	common->wcmTPCButtonDefault = 0; /* default Tablet PC button support is off */
 	common->wcmTPCButton = 
 		common->wcmTPCButtonDefault; /* set Tablet PC button on/off */
+	common->wcmTouch = 0;              /* touch is disabled */
+	common->wcmTouchDefault = 0; 	   /* default to disable when touch isn't supported */
+	common->wcmCapacity = -1;          /* Capacity is disabled */
+	common->wcmCapacityDefault = -1;    /* default to -1 when capacity isn't supported */
+					   /* 3 when capacity is supported */
 	common->wcmRotate = ROTATE_NONE;   /* default tablet rotation to off */
 	common->wcmMaxX = 0;               /* max digitizer logical X value */
 	common->wcmMaxY = 0;               /* max digitizer logical Y value */
 	common->wcmMaxTouchX = 1024;       /* max touch X value */
 	common->wcmMaxTouchY = 1024;       /* max touch Y value */
         common->wcmMaxZ = 0;               /* max Z value */
+        common->wcmMaxCapacity = 0;        /* max capacity value */
  	common->wcmMaxDist = 0;            /* max distance value */
 	common->wcmResolX = 0;             /* digitizer X resolution in points/inch */
 	common->wcmResolY = 0;             /* digitizer Y resolution in points/inch */
@@ -220,8 +217,8 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	common->wcmMaxCursorDist = 0;	/* Max distance received so far */
 	common->wcmCursorProxoutDist = 0;
 			/* Max mouse distance for proxy-out max/256 units */
-	common->wcmCursorProxoutDistDefault = PROXOUT_GRAPHIRE_DISTANCE; 
-			/* default to Graphire */
+	common->wcmCursorProxoutDistDefault = PROXOUT_INTUOS_DISTANCE; 
+			/* default to Intuos */
 	common->wcmSuppress = DEFAULT_SUPPRESS;    
 			/* transmit position if increment is superior */
 	common->wcmRawSample = DEFAULT_SAMPLES;    
@@ -372,13 +369,19 @@ static void xf86WcmUninit(InputDriverPtr drv, LocalDevicePtr local, int flags)
     
 	DBG(1, priv->debugLevel, ErrorF("xf86WcmUninit\n"));
 
+#ifndef WCM_XORG_XSERVER_1_4
 	gWacomModule.DevProc(local->dev, DEVICE_OFF);
+#endif
 
 	/* free pressure curve */
 	if (priv->pPressCurve)
 		xfree(priv->pPressCurve);
     
+	/* free priv here otherwise X server 1.6 or later crashes 
+	 */
 	xfree(priv);
+	local->private = NULL;
+
 	xf86DeleteInput(local, 0);    
 }
 
@@ -582,6 +585,7 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 		common->wcmDevCls = &gWacomISDV4Device;
 		xf86Msg(X_CONFIG, "%s: forcing TabletPC ISD V4 protocol\n",
 			dev->identifier);
+		common->wcmTPCButtonDefault = 1; /* Tablet PC buttons on by default */
 	}
 
 	s = xf86FindOptionValue(local->options, "Rotate");
@@ -779,18 +783,18 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 			common->wcmThreshold);
 
 	priv->wcmMaxX = xf86SetIntOption(local->options, "MaxX",
-		priv->wcmMaxX);
-	if (priv->wcmMaxX != 0)
-		xf86Msg(X_CONFIG, "%s: max x = %d\n", dev->identifier,
+		common->wcmMaxX);
+	if (priv->wcmMaxX > 0)
+		xf86Msg(X_CONFIG, "%s: max x set to %d by xorg.conf\n", dev->identifier,
 			priv->wcmMaxX);
 
 	/* Update tablet logical max X */
 	if (!IsTouch(priv)) common->wcmMaxX = priv->wcmMaxX;
 
 	priv->wcmMaxY = xf86SetIntOption(local->options, "MaxY",
-		priv->wcmMaxY);
-	if (priv->wcmMaxY != 0)
-		xf86Msg(X_CONFIG, "%s: max y = %d\n", dev->identifier,
+		common->wcmMaxY);
+	if (priv->wcmMaxY > 0)
+		xf86Msg(X_CONFIG, "%s: max y set to %d by xorg.conf\n", dev->identifier,
 			priv->wcmMaxY);
 
 	/* Update tablet logical max Y */
@@ -827,13 +831,24 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	}
 
 	/* Tablet PC button applied to the whole tablet. Not just one tool */
-	if ( !common->wcmTPCButton )
+	if ( priv->flags & STYLUS_ID )
 	{
-		common->wcmTPCButton = xf86SetBoolOption(local->options, "TPCButton", 0);
+		common->wcmTPCButton = xf86SetBoolOption(local->options, 
+			"TPCButton", common->wcmTPCButtonDefault);
 		if ( common->wcmTPCButton )
-			xf86Msg(X_CONFIG, "%s: Tablet PC buttons on \n", common->wcmDevice);
+			xf86Msg(X_CONFIG, "%s: Tablet PC buttons are on \n", common->wcmDevice);
 	}
 
+	/* Touch applies to the whole tablet */
+	common->wcmTouch = xf86SetBoolOption(local->options, "Touch", common->wcmTouchDefault);
+	if ( common->wcmTouch )
+		xf86Msg(X_CONFIG, "%s: Touch is enabled \n", common->wcmDevice);
+
+	/* Touch capacity applies to the whole tablet */
+	common->wcmCapacity = xf86SetBoolOption(local->options, "Capacity", common->wcmCapacityDefault);
+	if ( common->wcmCapacity >= 0 )
+		xf86Msg(X_CONFIG, "%s: Touch capacity is enabled \n", common->wcmDevice);
+
 	/* Mouse cursor stays in one monitor in a multimonitor setup */
 	if ( !priv->wcmMMonitor )
 	{
@@ -900,13 +915,22 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	if (s) xf86Msg(X_CONFIG, "%s: Twinview = %s\n", dev->identifier, s);
 	if (s && xf86NameCmp(s, "none") == 0) 
 		priv->twinview = TV_NONE;
-	else if (s && xf86NameCmp(s, "horizontal") == 0) 
+	else if ((s && xf86NameCmp(s, "horizontal") == 0) ||
+			(s && xf86NameCmp(s, "rightof") == 0)) 
 		priv->twinview = TV_LEFT_RIGHT;
-	else if (s && xf86NameCmp(s, "vertical") == 0) 
+	else if ((s && xf86NameCmp(s, "vertical") == 0) ||
+			(s && xf86NameCmp(s, "belowof") == 0)) 
 		priv->twinview = TV_ABOVE_BELOW;
+	else if (s && xf86NameCmp(s, "leftof") == 0) 
+		priv->twinview = TV_RIGHT_LEFT;
+	else if (s && xf86NameCmp(s, "aboveof") == 0) 
+		priv->twinview = TV_BELOW_ABOVE;
+	else if (s && xf86NameCmp(s, "xinerama") == 0) 
+		priv->twinview = TV_XINERAMA;
 	else if (s) 
 	{
-		xf86Msg(X_ERROR, "%s: invalid Twinview (should be none, vertical or horizontal). Using none.\n",
+		xf86Msg(X_ERROR, "%s: invalid Twinview (should be none, vertical (belowof), "
+			"horizontal (rightof), aboveof, leftof, xinerama). Using none.\n",
 			dev->identifier);
 		priv->twinview = TV_NONE;
 	}
@@ -975,7 +999,11 @@ static XF86ModuleVersionInfo xf86WcmVersionRec =
 	MODULEVENDORSTRING,
 	MODINFOSTRING1,
 	MODINFOSTRING2,
+#ifdef WCM_XORG_XSERVER_1_6
+	XORG_VERSION_CURRENT,
+#else
 	XF86_VERSION_CURRENT,
+#endif
 	1, 0, 0,
 	ABI_CLASS_XINPUT,
 	ABI_XINPUT_VERSION,
@@ -983,7 +1011,11 @@ static XF86ModuleVersionInfo xf86WcmVersionRec =
 	{0, 0, 0, 0}  /* signature, to be patched into the file by a tool */
 };
 
-XF86ModuleData wacomModuleData =
+#ifdef _X_EXPORT
+    _X_EXPORT XF86ModuleData wacomModuleData =
+#else
+    XF86ModuleData wacomModuleData =
+#endif
 {
 	&xf86WcmVersionRec,
 	xf86WcmPlug,
