@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org> 
- * Copyright 2002-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
+ * Copyright 2002-2009 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -68,9 +68,22 @@
  * 2008-02-27 47-pc0.7.9-8 - Support Cintiq 20
  * 2008-03-07 47-pc0.7.9-9 - Support keystrokes in wacomcpl
  * 2008-04-07 47-pc0.7.9-11 - Synchronized databases
+ * 2008-05-06 47-pc0.8.0-1 - new release
+ * 2008-05-14 47-pc0.8.0-2 - Update rotation routine
+ * 2008-07-09 47-pc0.8.1   - new release
+ * 2008-07-17 47-pc0.8.1-1 - Support USB TabletPC
+ * 2008-08-27 47-pc0.8.1-4 - Support Bamboo1 Meadium and Monarch
+ * 2008-11-11 47-pc0.8.2 - new release
+ * 2008-12-22 47-pc0.8.2-1 - fixed a few issues
+ * 2009-03-26 47-pc0.8.3 - Added Intuos4 support
+ * 2009-04-03 47-pc0.8.3-2 - HAL support
+ * 2009-05-08 47-pc0.8.3-4 - Fixed a pad button issue
+ * 2009-05-22 47-pc0.8.3-5 - Support Nvidia Xinerama
+ * 2009-06-26 47-pc0.8.3-6 - Support DTF720a
+ * 2009-07-14 47-pc0.8.3-7 - Support Nvidia Xinerama setting
  */
 
-static const char identification[] = "$Identification: 47-0.7.9-11 $";
+static const char identification[] = "$Identification: 47-0.8.3-7 $";
 
 /****************************************************************************/
 
@@ -90,6 +103,7 @@ extern Bool usbWcmInit(LocalDevicePtr pDev);
 extern int usbWcmGetRanges(LocalDevicePtr local);
 extern int xf86WcmDevChangeControl(LocalDevicePtr local, xDeviceCtl* control);
 extern int xf86WcmDevSwitchMode(ClientPtr client, DeviceIntPtr dev, int mode);
+extern void xf86WcmRotateTablet(LocalDevicePtr local, int value);
 extern void xf86WcmInitialScreens(LocalDevicePtr local);
 
 WacomModule gWacomModule =
@@ -121,6 +135,38 @@ static void xf86WcmKbdCtrlCallback(DeviceIntPtr di, KeybdCtrl* ctrl)
 }
 #endif /* WCM_KEY_SENDING_SUPPORT */
 
+/*****************************************************************************
+ * xf86WcmDesktopSize --
+ *   calculate the whole desktop size 
+ ****************************************************************************/
+static void xf86WcmDesktopSize(LocalDevicePtr local)
+{
+	WacomDevicePtr priv = (WacomDevicePtr) local->private;
+	int i = 0, minX = 0, minY = 0, maxX = 0, maxY = 0;
+
+	xf86WcmInitialScreens(local);
+	minX = priv->screenTopX[0];
+	minY = priv->screenTopY[0];
+	maxX = priv->screenBottomX[0];
+	maxY = priv->screenBottomY[0];
+	if (priv->numScreen != 1)
+	{
+		for (i = 1; i < priv->numScreen; i++)
+		{
+			if (priv->screenTopX[i] < minX)
+				minX = priv->screenTopX[i];
+			if (priv->screenTopY[i] < minY)
+				minY = priv->screenTopY[i];
+			if (priv->screenBottomX[i] > maxX)
+				maxX = priv->screenBottomX[i];
+			if (priv->screenBottomY[i] > maxY)
+				maxY = priv->screenBottomY[i];
+		}
+	}
+	priv->maxWidth = maxX - minX;
+	priv->maxHeight = maxY - minY;
+} 
+
 static int xf86WcmInitArea(LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
@@ -155,13 +201,13 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 		area->bottomY = priv->bottomY = priv->wcmMaxY;
 	}
 
-	if (priv->twinview != TV_NONE)
+	if (priv->twinview > TV_XINERAMA)
 		priv->numScreen = 2;
 
 	if (priv->screen_no != -1 &&
 		(priv->screen_no >= priv->numScreen || priv->screen_no < 0))
 	{
-		if (priv->twinview == TV_NONE || priv->screen_no != 1)
+		if (priv->twinview <= TV_XINERAMA)
 		{
 			ErrorF("%s: invalid screen number %d, resetting to default (-1) \n",
 					local->name, priv->screen_no);
@@ -170,9 +216,11 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 	}
 
 	/* need maxWidth and maxHeight for keepshape */
-	xf86WcmMappingFactor(local);
+	xf86WcmDesktopSize(local);
 
-	/* Maintain aspect ratio */
+	/* Maintain aspect ratio to the whole desktop
+	 * May need to consider a specific screen in multimonitor settings
+	 */
 	if (priv->flags & KEEP_SHAPE_FLAG)
 	{
 
@@ -195,8 +243,6 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 				screenRatio / tabletRatio + priv->topX;
 			area->bottomY = priv->bottomY = priv->wcmMaxY;
 		}
-		/* active tablet size has been changed */
-		xf86WcmMappingFactor(local);
 	}
 	/* end keep shape */ 
 
@@ -207,7 +253,7 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 	{
 		inlist = priv->tool->arealist;
 
-		/* remove this area from the list */
+		/* remove this overlapped area from the list */
 		for (; inlist; inlist=inlist->next)
 		{
 			if (inlist->next == area)
@@ -246,6 +292,78 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 	return TRUE;
 }
 
+/*****************************************************************************
+ * xf86WcmVirtaulTabletPadding(LocalDevicePtr local)
+ ****************************************************************************/
+
+void xf86WcmVirtaulTabletPadding(LocalDevicePtr local)
+{
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	int i;
+
+	priv->leftPadding = 0;
+	priv->topPadding = 0;
+
+	if (!(priv->flags & ABSOLUTE_FLAG)) return;
+
+	if ((priv->screen_no != -1) || (priv->twinview > TV_XINERAMA) || (!priv->wcmMMonitor))
+	{
+		i = priv->currentScreen;
+
+		priv->leftPadding = priv->bottomX - priv->topX -priv->tvoffsetX;
+ 		priv->topPadding = priv->bottomY - priv->topY - priv->tvoffsetY;
+
+		priv->leftPadding = (int)(((double)priv->screenTopX[i] * priv->leftPadding )
+			/ ((double)(priv->screenBottomX[i] - priv->screenTopX[i])) + 0.5);
+
+		priv->topPadding = (int)((double)(priv->screenTopY[i] * priv->topPadding)
+			/ ((double)(priv->screenBottomY[i] - priv->screenTopY[i])) + 0.5);
+	}
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmVirtaulTabletPadding for \"%s\" "
+		"x=%d y=%d \n", local->name, priv->leftPadding, priv->topPadding));
+	return;
+}
+
+/*****************************************************************************
+ * xf86WcmVirtaulTabletSize(LocalDevicePtr local)
+ ****************************************************************************/
+
+void xf86WcmVirtaulTabletSize(LocalDevicePtr local)
+{
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	int i, tabletSize;
+
+	if (!(priv->flags & ABSOLUTE_FLAG))
+	{
+		priv->sizeX = priv->bottomX - priv->topX;
+		priv->sizeY = priv->bottomY - priv->topY;
+		return;
+	}
+
+	priv->sizeX = priv->bottomX - priv->topX - priv->tvoffsetX;
+	priv->sizeY = priv->bottomY - priv->topY - priv->tvoffsetY;
+
+	if ((priv->screen_no != -1) || (priv->twinview > TV_XINERAMA) || (!priv->wcmMMonitor))
+	{
+		i = priv->currentScreen;
+
+		tabletSize = priv->sizeX;
+		priv->sizeX += (int)(((double)priv->screenTopX[i] * tabletSize)
+			/ ((double)(priv->screenBottomX[i] - priv->screenTopX[i])) + 0.5);
+		priv->sizeX += (int)((double)((priv->maxWidth - priv->screenBottomX[i])
+			* tabletSize) / ((double)(priv->screenBottomX[i] - priv->screenTopX[i])) + 0.5);
+
+		tabletSize = priv->sizeY;
+		priv->sizeY += (int)((double)(priv->screenTopY[i] * tabletSize)
+			/ ((double)(priv->screenBottomY[i] - priv->screenTopY[i])) + 0.5);
+		priv->sizeY += (int)((double)((priv->maxHeight - priv->screenBottomY[i])
+			* tabletSize) / ((double)(priv->screenBottomY[i] - priv->screenTopY[i])) + 0.5);
+	}
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmVirtaulTabletSize for \"%s\" "
+		"x=%d y=%d \n", local->name, priv->sizeX, priv->sizeY));
+	return;
+}
+
 /*****************************************************************************
  * xf86WcmInitialCoordinates
  ****************************************************************************/
@@ -253,64 +371,61 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
-	int tabletSize = 0, topx = 0, topy = 0;
-	int resolution;
+	int topx = 0, topy = 0, resolution;
+	int bottomx = priv->wcmMaxX, bottomy = priv->wcmMaxY;
+
+	xf86WcmMappingFactor(local);
 
 	/* x ax */
 	if ( !axes )
 	{
-		if (priv->twinview == TV_LEFT_RIGHT)
-			tabletSize = 2*(priv->bottomX - priv->topX - 2*priv->tvoffsetX);
-		else
-		{
-			if (priv->flags & ABSOLUTE_FLAG)
-				tabletSize = priv->bottomX;
-			else
-				tabletSize = priv->bottomX - priv->topX;
-		}
 		if (priv->flags & ABSOLUTE_FLAG)
-			topx = priv->topX - priv->tvoffsetX;
+		{
+			topx = priv->topX;
+			bottomx = priv->sizeX + priv->topX;
+			if ((priv->currentScreen == 1) && (priv->twinview > TV_XINERAMA))
+				topx += priv->tvoffsetX;
+			if ((priv->currentScreen == 0) && (priv->twinview > TV_XINERAMA))
+				bottomx -= priv->tvoffsetX;
+		}
 
 		resolution = priv->wcmResolX;
 #ifdef WCM_XORG_TABLET_SCALING
 		/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
 		 * for coordinate conversion at the moment */
-		if (priv->flags & ABSOLUTE_FLAG) tabletSize -= topx;
 		topx = 0;
-		tabletSize = (int)((double)tabletSize * priv->factorX + 0.5);
+		bottomx = (int)((double)priv->sizeX * priv->factorX + 0.5);
 		resolution = (int)((double)resolution * priv->factorX + 0.5);
 #endif
 
-		InitValuatorAxisStruct(local->dev, 0, topx, tabletSize, 
+		InitValuatorAxisStruct(local->dev, 0, topx, bottomx, 
 			resolution, 0, resolution); 
 	}
 	else /* y ax */
 	{
-		if (priv->twinview == TV_ABOVE_BELOW)
-			tabletSize = 2*(priv->bottomY - priv->topY - 2*priv->tvoffsetY);
-		else
-		{
-			if (priv->flags & ABSOLUTE_FLAG)
-				tabletSize = priv->bottomY;
-			else
-				tabletSize = priv->bottomY - priv->topY;
-		}
 		if (priv->flags & ABSOLUTE_FLAG)
-			topy = priv->topY - priv->tvoffsetY;
+		{
+			topy = priv->topY;
+			bottomy = priv->sizeY + priv->topY;
+			if ((priv->currentScreen == 1) && (priv->twinview > TV_XINERAMA))
+				topy += priv->tvoffsetY;
+			if ((priv->currentScreen == 0) && (priv->twinview > TV_XINERAMA))
+				bottomy -= priv->tvoffsetY;
+		}
 
 		resolution = priv->wcmResolY;
 #ifdef WCM_XORG_TABLET_SCALING
 		/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
 		 * for coordinate conversion at the moment */
-		if (priv->flags & ABSOLUTE_FLAG) tabletSize -= topy;
 		topy = 0;
-		tabletSize = (int)((double)tabletSize * priv->factorY + 0.5);
+		bottomy = (int)((double)priv->sizeY * priv->factorY + 0.5);
 		resolution = (int)((double)resolution * priv->factorY + 0.5);
 #endif
 
-		InitValuatorAxisStruct(local->dev, 1, topy, tabletSize, 
+		InitValuatorAxisStruct(local->dev, 1, topy, bottomy, 
 			resolution, 0, resolution); 
 	}
+	return;
 }
 
 #ifdef WCM_KEY_SENDING_SUPPORT
@@ -474,7 +589,6 @@ static void xf86WcmInitialToolSize(LocalDevicePtr local)
 	WacomCommonPtr common = priv->common;
 	WacomToolPtr toollist = common->wcmTool;
 	WacomToolAreaPtr arealist;
-	int temp;
 
 	if (IsTouch(priv))
 	{
@@ -491,14 +605,6 @@ static void xf86WcmInitialToolSize(LocalDevicePtr local)
 		priv->wcmResolY = common->wcmResolY;
 	}
 
-	/* Rotation rotates the Max Y and Y */
-	if (common->wcmRotate==ROTATE_CW || common->wcmRotate==ROTATE_CCW)
-	{
-		temp = priv->wcmMaxX;
-		priv->wcmMaxX = priv->wcmMaxY;
-		priv->wcmMaxY = temp;
-	}
-
 	for (; toollist; toollist=toollist->next)
 	{
 		arealist = toollist->arealist;
@@ -510,6 +616,7 @@ static void xf86WcmInitialToolSize(LocalDevicePtr local)
 				arealist->bottomY = priv->wcmMaxY;
 		}
 	}
+
 	return;
 }
 
@@ -532,8 +639,14 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 
 	nbaxes = priv->naxes;       /* X, Y, Pressure, Tilt-X, Tilt-Y, Wheel */
 	nbbuttons = priv->nbuttons; /* Use actual number of buttons, if possible */
-	nbkeys = nbbuttons;         /* Same number of keys since any button may be */
-	                            /* configured as an either mouse button or key */
+	nbkeys = nbbuttons;         /* Same number of keys since any button may be 
+	                             * configured as an either mouse button or key */
+
+	if (!nbbuttons)
+		nbbuttons = nbkeys = 1;	    /* Xserver 1.5 or later crashes when 
+			            	     * nbbuttons = 0 while sending a beep 
+			             	     * This is only a workaround. 
+				     	     */
 
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmRegisterX11Devices "
 		"(%s) %d buttons, %d keys, %d axes\n",
@@ -542,16 +655,6 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 		IsPad(priv) ? "pad" : "eraser",
 		nbbuttons, nbkeys, nbaxes));
 
-	xf86WcmInitialToolSize(local);
-
-	/* initialize screen bounding rect */
-	xf86WcmInitialScreens(local);
-
-	if (xf86WcmInitArea(local) == FALSE)
-	{
-		return FALSE;
-	}
-
 	for(loop=1; loop<=nbbuttons; loop++)
 		butmap[loop] = loop;
 
@@ -580,7 +683,7 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 			return FALSE;
 	}
 
-	if (nbaxes || nbaxes > 6)
+	if (!nbaxes || nbaxes > 6)
 		nbaxes = priv->naxes = 6;
 
 	if (InitValuatorClassDeviceStruct(local->dev, nbaxes,
@@ -588,7 +691,9 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 					  xf86GetMotionEvents,
 					  local->history_size,
 #else
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 3
 					  GetMotionHistory,
+#endif
 					  GetMotionHistorySize(),
 #endif
 					  ((priv->flags & ABSOLUTE_FLAG) ?
@@ -662,11 +767,15 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 	xf86MotionHistoryAllocate(local);
 #endif
 
-	/* x */
-	xf86WcmInitialCoordinates(local, 0);
+ 	xf86WcmInitialToolSize(local);
 
-	/* y */
-	xf86WcmInitialCoordinates(local, 1);
+	if (xf86WcmInitArea(local) == FALSE)
+	{
+		return FALSE;
+	}
+
+	/* Rotation rotates the Max X and Y */
+	xf86WcmRotateTablet(local, common->wcmRotate);
 
 	/* pressure */
 	InitValuatorAxisStruct(local->dev, 2, 0, 
@@ -681,7 +790,8 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 	else if (IsPad(priv))
 	{
 		/* strip-x and strip-y */
-		if (priv->naxes)
+		if (strstr(common->wcmModel->name, "Intuos3") || 
+			strstr(common->wcmModel->name, "CintiqV5")) 
 		{
 			InitValuatorAxisStruct(local->dev, 3, 0, common->wcmMaxStripX, 1, 1, 1);
 			InitValuatorAxisStruct(local->dev, 4, 0, common->wcmMaxStripY, 1, 1, 1);
@@ -695,11 +805,15 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 	}
 
 	if ((strstr(common->wcmModel->name, "Intuos3") || 
-		strstr(common->wcmModel->name, "CintiqV5")) 
+		strstr(common->wcmModel->name, "CintiqV5") ||
+		strstr(common->wcmModel->name, "Intuos4")) 
 			&& IsStylus(priv))
 		/* Art Marker Pen rotation */
 		InitValuatorAxisStruct(local->dev, 5, -900, 899, 1, 1, 1);
-	else if (strstr(common->wcmModel->name, "Bamboo") && IsPad(priv))
+	else if ((strstr(common->wcmModel->name, "Bamboo") ||
+		strstr(common->wcmModel->name, "Intuos4"))
+			&& IsPad(priv))
+		/* Touch ring */
 		InitValuatorAxisStruct(local->dev, 5, 0, 71, 1, 1, 1);
 	else
 	{
@@ -707,6 +821,12 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 		InitValuatorAxisStruct(local->dev, 5, 0, 1023, 1, 1, 1);
 	}
 
+	if (IsTouch(priv))
+	{
+		/* hard prox out */
+		priv->hardProx = 0;
+	}
+
 	return TRUE;
 }
 
@@ -758,7 +878,7 @@ char *xf86WcmEventAutoDevProbe (LocalDevicePtr local)
 		ErrorF("%s waiting 100 msec (total %dms) for device to become ready\n", local->name, wait); 
 		usleep(100*1000);
 	}
-	ErrorF("%s no synaptics event device found (checked %d nodes, waited %d msec)\n",
+	ErrorF("%s no Wacom event device found (checked %d nodes, waited %d msec)\n",
 		local->name, i + 1, wait);
 	return FALSE;
 }
@@ -837,11 +957,11 @@ static void xf86WcmDevReadInput(LocalDevicePtr local)
 	/* move data until we exhaust the device */
 	for (loop=0; loop < MAX_READ_LOOPS; ++loop)
 	{
+		/* verify that there is still data in pipe */
+		if (!xf86WcmReady(local)) break;
+
 		/* dispatch */
 		common->wcmDevCls->Read(local);
-
-		/* verify that there is still data in pipe */
-		if (!xf86WcmReady(local->fd)) break;
 	}
 
 	/* report how well we're doing */
@@ -856,6 +976,7 @@ void xf86WcmReadPacket(LocalDevicePtr local)
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 	int len, pos, cnt, remaining;
+	unsigned char * data;
 
  	DBG(10, common->debugLevel, ErrorF("xf86WcmReadPacket: device=%s"
 		" fd=%d \n", common->wcmDevice, local->fd));
@@ -891,7 +1012,22 @@ void xf86WcmReadPacket(LocalDevicePtr local)
 
 	pos = 0;
 
-	/* while there are whole packets present, parse data */
+	/* while there are whole packets present, check the packet length
+	 * for serial ISDv4 packet since it's different for pen and touch
+	 */
+	if (common->wcmForceDevice == DEVICE_ISDV4 && common->wcmDevCls != &gWacomUSBDevice) 
+	{
+		common->wcmPktLength = 9;
+		data = common->buffer;
+		if ( data[0] & 0x18 )
+		{
+			if (common->wcmMaxCapacity)
+				common->wcmPktLength = 7;
+			else
+				common->wcmPktLength = 5;
+		}
+	}
+
 	while ((common->bufpos - pos) >=  common->wcmPktLength)
 	{
 		/* parse packet */
@@ -902,6 +1038,29 @@ void xf86WcmReadPacket(LocalDevicePtr local)
 			break;
 		}
 		pos += cnt;
+
+		if (common->wcmDevCls != &gWacomUSBDevice) 
+		{
+			data = common->buffer + pos;
+			if ( data[0] & 0x18 )
+			{
+				if (common->wcmPktLength == 9)
+				{
+					DBG(1, common->debugLevel, 
+						ErrorF("xf86WcmReadPacket: not a pen data any more \n"));
+					break;	
+				}
+			}
+			else
+			{
+				if (common->wcmPktLength != 9)
+				{
+					DBG(1, common->debugLevel, 
+						ErrorF("xf86WcmReadPacket: not a touch data any more \n"));
+					break;	
+				}
+			}
+		}
 	}
  
 	if (pos)
@@ -1034,7 +1193,6 @@ static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
 		int v0, int v1, int v2, int v3, int v4, int v5, int* x, int* y)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
-	double temp;
     
 	DBG(6, priv->debugLevel, ErrorF("xf86WcmDevConvert v0=%d v1=%d on screen %d \n",
 		 v0, v1, priv->currentScreen));
@@ -1042,62 +1200,37 @@ static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
 	if (first != 0 || num == 1) 
  		return FALSE;
 
-	*x = 0;
-	*y = 0;
-
 	if (priv->flags & ABSOLUTE_FLAG)
 	{
-		int leftPadding = 0;
-		int topPadding = 0;				
-
-		v0 = v0 - priv->topX - priv->tvoffsetX;
-		v1 = v1 - priv->topY - priv->tvoffsetY;
-
-		if (priv->twinview == TV_NONE)
+		v0 -= priv->topX;
+		v1 -= priv->topY;
+		if ((priv->currentScreen == 1) && (priv->twinview > TV_XINERAMA))
 		{
-			if (priv->screen_no == -1)
-			{
-				leftPadding = priv->screenTopX[priv->currentScreen];
-				topPadding = priv->screenTopY[priv->currentScreen];				
-			}
-			*x = - leftPadding;
-			*y = - topPadding;
+			v0 -= priv->tvoffsetX;
+			v1 -= priv->tvoffsetY;
 		}
-		else
-		{
-			*x = priv->screenTopX[priv->currentScreen];
-			*y = priv->screenTopY[priv->currentScreen];
-		}
-	}
-	temp = ((double)v0 * priv->factorX + 0.5);
-	*x += temp;
-	temp = ((double)v1 * priv->factorY + 0.5);
-	*y += temp;
+ 	}
 
-	DBG(6, priv->debugLevel, ErrorF("xf86WcmDevConvert v0=%d v1=%d to x=%d y=%d\n", v0, v1, *x, *y));
-	if ((priv->screen_no != -1 || !priv->wcmMMonitor) && (priv->flags & ABSOLUTE_FLAG))
+	*x = (double)v0 * priv->factorX + 0.5;
+	*y = (double)v1 * priv->factorY + 0.5;
+
+	if ((priv->flags & ABSOLUTE_FLAG) && (priv->twinview <= TV_XINERAMA))
 	{
-		DBG(6, priv->debugLevel, ErrorF("xf86WcmDevConvert restricted (%d,%d)", *x, *y));
-		if (priv->twinview == TV_NONE)
-		{
-			if (*x < 1) *x = 0;
-			if (*y < 1) *y = 0;
-			if (*x >= priv->screenBottomX[priv->currentScreen] - priv->screenTopX[priv->currentScreen])
-				*x = priv->screenBottomX[priv->currentScreen] - priv->screenTopX[priv->currentScreen]-1;
-			if (*y >= priv->screenBottomY[priv->currentScreen] - priv->screenTopY[priv->currentScreen])
-				*y = priv->screenBottomY[priv->currentScreen] - priv->screenTopY[priv->currentScreen]-1;
-		}
-		else
-		{
-			if (*x < priv->screenTopX[priv->currentScreen]+1) *x = priv->screenTopX[priv->currentScreen];
-			if (*y < priv->screenTopY[priv->currentScreen]+1) *y = priv->screenTopY[priv->currentScreen];
-			if (*x >= priv->screenBottomX[priv->currentScreen])
-				*x = priv->screenBottomX[priv->currentScreen]-1;
-			if (*y >= priv->screenBottomY[priv->currentScreen])
-				*y = priv->screenBottomY[priv->currentScreen]-1;
-		}
-		DBG(6, priv->debugLevel, ErrorF(" to x=%d y=%d\n", *x, *y));
+		*x -= priv->screenTopX[priv->currentScreen];
+		*y -= priv->screenTopY[priv->currentScreen];
 	}
+
+	if (priv->screen_no != -1)
+	{
+		if (*x > priv->screenBottomX[priv->currentScreen] - priv->screenTopX[priv->currentScreen])
+			*x = priv->screenBottomX[priv->currentScreen];
+		if (*x < 0) *x = 0;
+		if (*y > priv->screenBottomY[priv->currentScreen] - priv->screenTopY[priv->currentScreen])
+			*y = priv->screenBottomY[priv->currentScreen];
+		if (*y < 0) *y = 0;
+	
+	}
+	DBG(6, priv->debugLevel, ErrorF("xf86WcmDevConvert v0=%d v1=%d to x=%d y=%d\n", v0, v1, *x, *y));
 	return TRUE;
 }
 
@@ -1134,6 +1267,7 @@ static Bool xf86WcmDevReverseConvert(LocalDevicePtr local, int x, int y,
 		else
 			priv->devReverseCount = 0;
 	}
+
 	DBG(6, priv->debugLevel, ErrorF("Wacom converted x=%d y=%d"
 		" to v0=%d v1=%d v2=%d v3=%d v4=%d v5=%d\n", x, y,
 		valuators[0], valuators[1], valuators[2], 
