@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
+ * Copyright 2002-2009 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -161,28 +161,30 @@ struct _WacomDeviceRec
 	int screenBottomY[32];	/* bottom cordinate of the associated screen */
 	int maxWidth;		/* max active screen width */
 	int maxHeight;		/* max active screen height */
+	int leftPadding;	/* left padding for virtual tablet */
+	int topPadding;		/* top padding for virtual tablet */
 	int button[MAX_BUTTONS];/* buttons assignments */
 	unsigned keys[MAX_BUTTONS][256]; /* keystrokes assigned to buttons */
 	int relup;
-	unsigned rupk[256]; /* keystrokes assigned to relative wheel up event (default is button 4) */
+	unsigned rupk[256];     /* keystrokes assigned to relative wheel up event (default is button 4) */
 	int reldn;
-	unsigned rdnk[256]; /* keystrokes assigned to relative wheel down event (default is button 5) */
+	unsigned rdnk[256];     /* keystrokes assigned to relative wheel down event (default is button 5) */
 	int wheelup;
-	unsigned wupk[256]; /* keystrokes assigned to absolute wheel or throttle up event (default is button 4) */
+	unsigned wupk[256];     /* keystrokes assigned to absolute wheel/throttle up event (default is button 4) */
 	int wheeldn;
-	unsigned wdnk[256]; /* keystrokes assigned to absolute wheel or throttle down event (default is button 5) */
+	unsigned wdnk[256];     /* keystrokes assigned to absolute wheel/throttle down event (default is button 5) */
 	int striplup;
-	unsigned slupk[256]; /* keystrokes assigned to left strip up event (default is button 4) */
+	unsigned slupk[256];    /* keystrokes assigned to left strip up event (default is button 4) */
 	int stripldn;
-	unsigned sldnk[256]; /* keystrokes assigned to left strip up event (default is button 5) */
+	unsigned sldnk[256];    /* keystrokes assigned to left strip up event (default is button 5) */
 	int striprup;
-	unsigned srupk[256]; /* keystrokes assigned to right strip up event (default is button 4) */
+	unsigned srupk[256];    /* keystrokes assigned to right strip up event (default is button 4) */
 	int striprdn;
- 	unsigned srdnk[256]; /* keystrokes assigned to right strip up event (default is button 4) */
-	int nbuttons;         /* number of buttons for this subdevice */
-	int naxes;            /* number of axes */
+ 	unsigned srdnk[256];    /* keystrokes assigned to right strip up event (default is button 4) */
+	int nbuttons;           /* number of buttons for this subdevice */
+	int naxes;              /* number of axes */
 
-	WacomCommonPtr common;/* common info pointer */
+	WacomCommonPtr common;  /* common info pointer */
 
 	/* state fields */
 	int currentX;           /* current X position */
@@ -192,6 +194,7 @@ struct _WacomDeviceRec
 	int oldX;               /* previous X position */
 	int oldY;               /* previous Y position */
 	int oldZ;               /* previous pressure */
+	int oldCapacity;        /* previous capacity */
 	int oldTiltX;           /* previous tilt in x direction */
 	int oldTiltY;           /* previous tilt in y direction */    
 	int oldWheel;           /* previous wheel value */    
@@ -223,8 +226,8 @@ struct _WacomDeviceRec
 	int throttleValue;      /* current throttle value */
 
 	/* JEJ - filters */
-	int* pPressCurve;               /* pressure curve */
-	int nPressCtrl[4];              /* control points for curve */
+	int* pPressCurve;       /* pressure curve */
+	int nPressCtrl[4];      /* control points for curve */
 
 	WacomToolPtr tool;         /* The common tool-structure for this device */
 	WacomToolAreaPtr toolarea; /* The area defined for this device */
@@ -260,6 +263,7 @@ struct _WacomDeviceState
 	int y;
 	int buttons;
 	int pressure;
+	int capacity;
 	int tiltx;
 	int tilty;
 	int stripx;
@@ -357,19 +361,19 @@ struct _WacomCommonRec
 	int fd_refs;                 /* number of references to fd; if =0, fd is invalid */
 
 	/* These values are in tablet coordinates */
-	int wcmMaxX;        /* tablet max X value */
-	int wcmMaxY;        /* tablet max Y value */
+	int wcmMaxX;                 /* tablet max X value */
+	int wcmMaxY;                 /* tablet max Y value */
 	int wcmMaxZ;                 /* tablet max Z value */
-	int wcmMaxTouchX;            /* max touch X value */
-	int wcmMaxTouchY;            /* max touch Y value */
-	int wcmMaxDist;              /* tablet max distance value */
-	int wcmTouchResolX;          /* touch X resolution in points/inch */
-	int wcmTouchResolY;          /* touch Y resolution in points/inch */
-	int wcmResolX;               /* tablet X resolution in points/inch */
-	int wcmResolY;               /* tablet Y resolution in points/inch */
+	int wcmMaxTouchX;            /* touch panel max X value */
+	int wcmMaxTouchY;            /* touch panel max Y value */
+	int wcmResolX;		     /* pen tool X resolution in points/inch */
+	int wcmResolY;		     /* pen tool Y resolution in points/inch */
+	int wcmTouchResolX;	     /* touch X resolution in points/inch */
+	int wcmTouchResolY;	     /* touch Y resolution in points/inch */
 	                             /* tablet Z resolution is equivalent
-	                              * to wcmMaxZ which is equal to 100%
-	                              * pressure */
+	                              * to wcmMaxZ which is equal to 100% pressure */
+	int wcmMaxCapacity;	     /* max capacity value */
+	int wcmMaxDist;              /* tablet max distance value */
 	int wcmMaxtiltX;	     /* styli max tilt in X directory */ 
 	int wcmMaxtiltY;	     /* styli max tilt in Y directory */ 
 
@@ -401,7 +405,12 @@ struct _WacomCommonRec
 	WacomModelPtr wcmModel;        /* model-specific functions */
 	char * wcmEraserID;	     /* eraser associated with the stylus */
 	int wcmTPCButton;	     /* set Tablet PC button on/off */
+	int wcmTouch;	             /* disable/enable touch event */
 	int wcmTPCButtonDefault;     /* Tablet PC button default */
+	int wcmTouchDefault;	     /* default to disable when not supported */
+	int wcmCapacity;	     /* disable/enable capacity */
+	int wcmCapacityDefault;      /* default to -1 when capacity isn't supported/disabled */
+				     /* 3 when capacity is supported */
 	int wcmMaxCursorDist;	     /* Max mouse distance reported so far */
 	int wcmCursorProxoutDist;    /* Max mouse distance for proxy-out max/256 units */
 	int wcmCursorProxoutDistDefault; /* Default max mouse distance for proxy-out */
